# chosung.py

import warnings
from hangulpy.utils import CHOSUNG_LIST, CHOSUNG_BASE, is_hangul_char, HANGUL_BEGIN_UNICODE

def get_chosung(c):
    """
    주어진 한글 음절의 초성을 반환합니다.
    
    :param c: 한글 음절 문자
    :return: 초성 문자 (한글이 아니면 그대로 반환)
    """
    warnings.warn(
        "The get_chosung function will be removed in a future version. Use the get_chosung_string function. Check out https://github.com/gaon12/hangulpy/issue/4 for more information!",
        DeprecationWarning,
        stacklevel=2
    )
    if is_hangul_char(c):
        char_index = ord(c) - HANGUL_BEGIN_UNICODE
        chosung_index = char_index // CHOSUNG_BASE
        return CHOSUNG_LIST[chosung_index]
    return c

def get_chosung_string(text, keep_spaces=False):
    """
    주어진 문자열의 각 문자의 초성을 반환합니다.
    
    :param text: 한글 문자열
    :param keep_spaces: 공백을 유지할지 여부 (기본값: False)
    :return: 초성 문자열
    """
    if keep_spaces:
        return ''.join(get_chosung(c) if is_hangul_char(c) else c for c in text)
    else:
        return ''.join(get_chosung(c) for c in text if is_hangul_char(c) or not c.isspace())

def chosungIncludes(word, pattern):
    """
    주어진 단어에 패턴의 초성이 포함되어 있는지 확인합니다.
    
    :param word: 검색할 단어
    :param pattern: 검색할 초성 패턴
    :return: 패턴이 단어의 초성에 포함되어 있으면 True, 아니면 False
    """
    # 단어의 각 문자를 초성으로 변환하여 문자열을 생성합니다.
    word_chosung = ''.join(get_chosung(c) for c in word)
    # 패턴이 초성 문자열에 포함되어 있는지 확인합니다.
    return pattern in word_chosung



# hangul_split.py

from hangulpy.utils import CHOSUNG_LIST, CHOSUNG_BASE, is_hangul_char, HANGUL_BEGIN_UNICODE, JUNGSUNG_LIST, JUNGSUNG_DECOMPOSE, JONGSUNG_LIST, JONGSUNG_DECOMPOSE, JUNGSUNG_BASE

def split_hangul_char(c):
    """
    주어진 한글 음절을 초성, 중성, 종성으로 분해하여 배열 형태로 반환합니다.
    
    :param c: 한글 음절 문자
    :return: 초성, 중성, 종성을 배열 형태로 반환
    """
    if is_hangul_char(c):
        # 한글 음절의 유니코드 값을 기준으로 각 성분의 인덱스를 계산합니다.
        char_index = ord(c) - HANGUL_BEGIN_UNICODE
        chosung_index = char_index // CHOSUNG_BASE
        jungsung_index = (char_index % CHOSUNG_BASE) // JUNGSUNG_BASE
        jongsung_index = char_index % JUNGSUNG_BASE

        # 중성 및 종성 분해
        jungsung = JUNGSUNG_LIST[jungsung_index]
        jongsung = JONGSUNG_LIST[jongsung_index]

        jungsung_decomposed = JUNGSUNG_DECOMPOSE.get(jungsung, [jungsung])
        jongsung_decomposed = JONGSUNG_DECOMPOSE.get(jongsung, [jongsung])

        return [CHOSUNG_LIST[chosung_index]] + list(jungsung_decomposed) + list(jongsung_decomposed)
    
    # 자모인 경우 추가 처리
    if c in JUNGSUNG_DECOMPOSE:
        return list(JUNGSUNG_DECOMPOSE[c])
    if c in JONGSUNG_DECOMPOSE:
        return list(JONGSUNG_DECOMPOSE[c])

    return [c]  # 한글이 아니면 그대로 반환

def split_hangul_string(s):
    """
    주어진 문자열의 각 한글 음절을 초성, 중성, 종성으로 분해하여 배열 형태로 반환합니다.
    
    :param s: 문자열
    :return: 각 한글 음절을 초성, 중성, 종성으로 분해한 결과를 포함하는 배열
    """
    result = []
    for char in s:
        result.extend(split_hangul_char(char))
    return result